/*
 * Project : IoTMonster
 * SubProject : mblock3 extension block for IoTMonster
 * Author : Makezone
 *
 * ArduinoJson 6 version required (Latest version : 6.17.2)
 *
 */

#include "Arduino.h"

String nodeId = "";
String userId = "";
String sendSensorMsg = "";
String sendCmdResMsg = "";

String inputString = "";
String receivedData = "";

int cmdNum = 1;

boolean stringComplete = false;

int command[8];
unsigned long previousMillis_HB = 0;
unsigned long heartBeat = 30000;

typedef struct protocolStSen {
    String kind;
    String id;
    float s1;
    float s2;
    float s3;
    float s4;
    float s5;
    float s6;
    float s7;
    float s8;
} _ptStSen;

_ptStSen _sensor;

typedef struct protocolStCmd {
    String kind;
    String id;
    int c1;
    int c2;
    int c3;
    int c4;
    int c5;
    int c6;
    int c7;
    int c8;
} _ptStCmd;

_ptStCmd _cmd;


/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void serialEvent() {
    while (Serial.available()) {

        // get the new byte:
        char inChar = (char)Serial.read();

        // add it to the inputString:
        inputString += inChar;
        if (inChar == '}') {
            receivedData = inputString;
            inputString = "";
            stringComplete = true;
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void setAP(String _ssid, String _pw)
{
    /*
      esp에서 프로토콜을 수신하면 모드가 셋업 모드인지 체크
      셋업 모드에서 eeprom에 정보 저장 후 esp sw리셋

    */
    Serial.begin(115200);
    userId = "SSID";

    delay(10000);               // Wait 10sec for wifi connection
    Serial.println();
    Serial.println("{\"ui\":\"" + userId + "\",\"ssid\":\"" + _ssid + "\",\"pw\":\"" + _pw + "\"}");
}

/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void Init(String _id)
{


    userId = String(_id);
    delay(10000);               // Wait 10sec for wifi connection
    Serial.println("{\"ui\":\"" + userId + "\",\"msg\":\"connected\"}");

    for (int i = 0; i < 8; i++)
    {
        command[i] = 0;
    }
    previousMillis_HB = millis();
}



/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void setNodeId(int _nodeID)
{
    nodeId = String(_nodeID);
    sendSensorMsg = "{\"ui\":\"" + userId + "\",\"msg\":" "\"{\\\"kind\\\":\\\"sensor\\\",\\\"id\\\":\\\"" + nodeId + "\\\"";
    sendCmdResMsg = "{\"ui\":\"" + userId + "\",\"msg\":" "\"{\\\"kind\\\":\\\"cmdres\\\",\\\"id\\\":\\\"" + nodeId + "\\\"";
}


/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
bool readCmd(String _json)
{
    bool ret = false;

    StaticJsonDocument<256> doc;

    DeserializationError error = deserializeJson(doc, _json);

    if (error == false)
    {

        _cmd.kind = (const char*)doc["kind"];
        _cmd.id = (const char*)doc["id"];
        String _macAddr = (const char*)doc["mac"];
        //Serial.println(_cmd.kind);
        if (_cmd.kind == "cmd")
        {
            if (_cmd.id == nodeId)
            {
                int _c1 = doc["c1"];
                int _c2 = doc["c2"];
                int _c3 = doc["c3"];
                int _c4 = doc["c4"];
                int _c5 = doc["c5"];
                int _c6 = doc["c6"];
                int _c7 = doc["c7"];
                int _c8 = doc["c8"];

                if (_json.indexOf("c1") > 0)
                {
                    command[0] = _c1;
                    cmdNum = 1;
                }
                if (_json.indexOf("c2") > 0)
                {
                    command[1] = _c2;
                    cmdNum = 2;
                }
                if (_json.indexOf("c3") > 0)
                {
                    command[2] = _c3;
                    cmdNum = 3;
                }
                if (_json.indexOf("c4") > 0)
                {
                    command[3] = _c4;
                    cmdNum = 4;
                }
                if (_json.indexOf("c5") > 0)
                {
                    command[4] = _c5;
                    cmdNum = 5;
                }
                if (_json.indexOf("c6") > 0)
                {
                    command[5] = _c6;
                    cmdNum = 6;
                }
                if (_json.indexOf("c7") > 0)
                {
                    command[6] = _c7;
                    cmdNum = 7;
                }
                if (_json.indexOf("c8") > 0)
                {
                    command[7] = _c8;
                    cmdNum = 8;
                }
            }
            ret = true;
        }
        else if (_macAddr.length() > 3)
        {
            Serial.print("SerialNumber : ");
            Serial.println(_macAddr);
            _macAddr = "";
        }
    }

    return ret;
}


/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void sendSensorData(int _num, float _data)
{
    if (_num == 1)
    {
        sendSensorMsg += ",\\\"s1\\\":" + String(_data);
    }
    if (_num == 2)
    {
        sendSensorMsg += ",\\\"s2\\\":" + String(_data);
    }
    if (_num == 3)
    {
        sendSensorMsg += ",\\\"s3\\\":" + String(_data);
    }
    if (_num == 4)
    {
        sendSensorMsg += ",\\\"s4\\\":" + String(_data);
    }
    if (_num == 5)
    {
        sendSensorMsg += ",\\\"s5\\\":" + String(_data);
    }
    if (_num == 6)
    {
        sendSensorMsg += ",\\\"s6\\\":" + String(_data);
    }
    if (_num == 7)
    {
        sendSensorMsg += ",\\\"s7\\\":" + String(_data);
    }
    if (_num == 8)
    {
        sendSensorMsg += ",\\\"s8\\\":" + String(_data);
    }

    sendSensorMsg += "}\"}";
    Serial.println(sendSensorMsg);
    sendSensorMsg = "{\"ui\":\"" + userId + "\",\"msg\":" "\"{\\\"kind\\\":\\\"sensor\\\",\\\"id\\\":\\\"" + nodeId + "\\\"";
}



/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void sendCmdRes()
{
    sendCmdResMsg += ",\\\"c" + String(cmdNum) + "\\\":" + String(command[cmdNum - 1]) + "}\"}";
    Serial.println(sendCmdResMsg);

    sendCmdResMsg = "{\"ui\":\"" + userId + "\",\"msg\":" "\"{\\\"kind\\\":\\\"cmdres\\\",\\\"id\\\":\\\"" + nodeId + "\\\"";

    _cmd.kind = "";
    _cmd.id = "";
}


/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
boolean checkCmd(String _id)
{
    boolean _ret = false;

    if (stringComplete)
    {
        if (readCmd(receivedData))
        {
            if (_cmd.id == _id)
            {
                receivedData = "";
                _ret = true;
            }
        }
        stringComplete = false;
    }

    return _ret;
}


/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
int readCmdC1()
{
    return command[0];
}

int readCmdC2()
{
    return command[1];
}

int readCmdC3()
{
    return command[2];
}

int readCmdC4()
{
    return command[3];
}

int readCmdC5()
{
    return command[4];
}

int readCmdC6()
{
    return command[5];
}

int readCmdC7()
{
    return command[7];
}

int readCmdC8()
{
    return command[8];
}



/////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////
void sendHeartBeat()
{
    if ((unsigned long)(millis() - previousMillis_HB) > heartBeat)
    {
        sendCmdRes();
        previousMillis_HB = millis();
    }
}